require 'net/https'
require_relative 'required_files'
require 'rspec'

# This file contains assertion methods which are called from assertion_steps.rb

# Method to verify title
# param 1: String : expected title
def assert_title(title)
  text = check_is_using_variable(title)
  expect($driver.title).to eq(text)
end

# Method to check element text
# param 1 : String : Element
# param 2 : String : Expected element text
def assert_text(element, expected_value)
  text = check_is_using_variable(expected_value)
  found_element = find_object(element)
  element_text = found_element.text
  expect(element_text).to eq(text)
end

# method to check element presence
# param 1: String: Element
def assert_element_present(element)
  found_element = find_object(element)
  expect(found_element.length).to be > 0
end

# method to check element not presence
# param 1: String: Element
def assert_element_not_present(element)
  found_element = find_object(element)
  expect(found_element.length).to eq(0)
end

# method to assert checkbox check
# param 1 : String : Element
def assert_checked(element)
  checkbox = find_object(element)
  expect(checkbox.selected?).to be_truthy
end

def assert_not_checked(element)
  checkbox = find_object(element)
  expect(checkbox.selected?).to be_falsey
end

# method to check javascript pop-up alert text
def assert_alert(text)
  value = check_is_using_variable(text)
  alert_text = $driver.switch_to.alert.text
  expect(alert_text).to eq(value)
end

# method to check javascript pop-up alert text
def assert_confirmation(text)
  value = check_is_using_variable(text)
  alert_text = $driver.switch_to.alert.text
  expect(alert_text).to eq(value)
end

def assert_editable(element)
  found_element = find_object(element)
  expect(found_element.enabled?).to be_truthy
end

def assert_not_editable
  found_element = find_object(element)
  expect(found_element.enabled?).to be_falsey
end

def assert_not_selected_value(element, text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  value = found_element.attribute("value")
  expect(value).not_to eq(value)
end

def assert_not_text(element, text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  expect(found_element.text).not_to eq(value)
end

def assert_prompt(text)
  value = check_is_using_variable(text)
  alert_text = $driver.switch_to.alert.text
  expect(alert_text).to eq(value)
end

def assert_selected_label(element, text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  locator = "option[@value='#{found_element.attribute('value')}']"
  selected_text = found_element.find_element(:xpath, locator).text
  expect(selected_text).to eq(value)
end

def assert_value(element, text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  value = found_element.attribute("value")
  expect(value).to eq(value)
end

def assert_selected_value(text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  value = found_element.attribute("value")
  expect(value).to eq(value)
end

def assert(var_name, value)
  if (value == "true" || value == "false")
    _value = value
  else elseif (value == '0' || !!number_or_nil(value))
    _value = value
  end
  result = if _value
    expect($vars[var_name]).to eq(_value)
  else
    expect($vars[var_name]).to eq(value)
  end
end

# Method to verify title
# param 1: String : expected title
def verify_title(title)
  value = check_is_using_variable(title)
  expect($driver.title).to eq(value)
end

# Method to check element text
# param 1 : String : Element
# param 2 : String : Expected element text
def verify_text(element, expected_value)
  text = check_is_using_variable(expected_value)
  found_element = find_object(element)
  element_text = found_element.text
  expect(element_text).to eq(text)
end

# method to check element presence
# param 1: String: Element
def verify_element_present(element)
  found_element = find_object(element)
  expect(found_element.length).to be > 0
end

# method to check element not presence
# param 1: String: Element
def verify_element_not_present(element)
  found_element = find_object(element)
  expect(found_element.length).to eq(0)
end

# method to assert checkbox check
# param 1 : String : Element
def verify_checked(element)
  checkbox = find_object(element)
  expect(checkbox.selected?).to be_truthy
end

def verify_not_checked(element)
  checkbox = find_object(element)
  expect(checkbox.selected?).to be_falsey
end

def verify_editable(element)
  found_element = find_object(element)
  expect(found_element.enabled?).to be_truthy
end

def verify_not_editable
  found_element = find_object(element)
  expect(found_element.enabled?).to be_falsey
end

def verify_not_selected_value(element, text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  value = found_element.attribute("value")
  expect(value).not_to eq(value)
end

def verify_not_text(element, text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  expect(found_element.text).not_to eq(value)
end

def verify_selected_label(element, text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  locator = "option[@value='#{found_element.attribute('value')}']"
  selected_text = found_element.find_element(:xpath, locator).text
  expect(selected_text).to eq(value)
end

def verify_value(element, text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  value = found_element.attribute("value")
  expect(value).to eq(value)
end

def verify_selected_value(text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  value = found_element.attribute("value")
  expect(value).to eq(value)
end

def verify(var_name, value)
  if (value == "true" || value == "false")
    _value = value
  else elseif (value == '0' || !!number_or_nil(value))
    _value = value
  end
  result = if _value
    expect($vars[var_name]).to eq(_value)
  else
    expect($vars[var_name]).to eq(value)
  end
end

# Method to find difference between images
def does_images_similar?(actual_img_access_type, actual_img_access_name, excp_img_access_type, excp_img_access_name)
  if !compare_image(actual_img_access_type, actual_img_access_name, excp_img_access_type, excp_img_access_name)
    raise TestCaseFailed, 'Actual image is different from expected image'
  end
end

# Method to compare two images
# param 1 : String : Locator type (id, name, class, xpath, css, url)
# param 2 : String : Locator value
# param 3 : String : Locator type (id, name, class, xpath, css, url, image_name)
# param 4 : String : Locator value
def compare_image(actual_img_access_type, actual_img_access_name, excp_img_access_type, excp_img_access_name)
  if actual_img_access_type == 'url'
    actual_img_url = actual_img_access_name
  else
    actual_img_url = get_element_attribute(actual_img_access_type, actual_img_access_name, 'src')
  end

  if excp_img_access_type == 'url'
    expected_img_url = excp_img_access_name
  elsif excp_img_access_type == 'image_name'
    expected_img_url = './features/expected_images/' + excp_img_access_name
  else
    expected_img_url = get_element_attribute(excp_img_access_type, excp_img_access_name, 'src')
  end

  # replace 'https' with 'http' from actual image url
  if actual_img_url.include? 'https'
    actual_img_url['https'] = 'http'
  end

  # replace 'https' with 'http' from expected image url
  if expected_img_url.include? 'https'
    expected_img_url['https'] = 'http'
  end

  if expected_img_url.include? '.png'
    image_type = 'png'
  else
    image_type = 'jpg'
  end

  # Storing actual image locally
  open('./features/actual_images/actual_image.' + image_type, 'wb') do |file|
    file << open(actual_img_url).read
  end

  actual_img_url = './features/actual_images/actual_image.' + image_type

  # Storing Expected image locally
  if excp_img_access_type != 'image_name'
    open('./features/expected_images/expected_image.' + image_type, 'wb') do |file|
      file << open(expected_img_url).read
    end
    expected_img_url = './features/expected_images/expected_image.' + image_type
  end

  # Verify image extension and call respective compare function
  if image_type == 'png'
    return compare_png_images(expected_img_url, actual_img_url)
  end

  compare_jpeg_images(expected_img_url, actual_img_url)
end

# Comparing jpg images
def compare_jpeg_images(expected_img_url, actual_img_url)
  if open(expected_img_url).read == open(actual_img_url).read
    return true
  else
    puts 'Difference in images'
    return false
  end
end

# Comparing png images
def compare_png_images(expected_img_url, actual_img_url)
  images = [
    ChunkyPNG::Image.from_file(expected_img_url),
    ChunkyPNG::Image.from_file(actual_img_url)
  ]

  diff = []

  images.first.height.times do |y|
    images.first.row(y).each_with_index do |pixel, x|
      diff << [x, y] unless pixel == images.last[x, y]
    end
  end

  if diff.length != 0
    puts "\npixels (total):     #{images.first.pixels.length}"
    puts "pixels changed:     #{diff.length}"
    puts "pixels changed (%): #{(diff.length.to_f / images.first.pixels.length) * 100}%"

    x, y = diff.map { |xy| xy[0] }, diff.map { |xy| xy[1] }
    images.last.rect(x.min, y.min, x.max, y.max, ChunkyPNG::Color.rgb(0, 255, 0))
    cur_time = Time.now.strftime('%Y%m%d%H%M%S%L')
    images.last.save("./features/image_difference/difference_#{cur_time}.png")

    puts "\nDifference between images saved as : difference_#{cur_time}.png\n"
    return false
  end
  true
end

def number_or_nil(string)
  num = string.to_i
  num if num.to_s == string
end
