require_relative 'required_files'

# method to enter text into textfield
def type(element, text)
  foundElement = find_object(element)
  foundElement.send_keys(text)
end

# method to send keys
def send_keys(element, key)
  value = check_is_using_variable(key)
  keys_support = ["KEY_LEFT", "KEY_UP", "KEY_RIGHT", "KEY_DOWN", "KEY_PAGE_UP", "KEY_PAGE_DOWN", "KEY_BACKSPACE", "KEY_DELETE", "KEY_ENTER", "KEY_TAB"]
  foundElement = find_object(element)
  if value
    get_key = value.strip[/^\$\{(.*?)\}$/, 1]
    raise TestCaseFailed, "${get_key} key is not supported. Supported keys: KEY_LEFT, KEY_UP, KEY_RIGHT, KEY_DOWN, KEY_PAGE_UP, KEY_PAGE_DOWN, KEY_BACKSPACE, KEY_DELETE, KEY_ENTER, KEY_TAB" unless keys_support.include?(get_key)
    key_convert = get_key[/^KEY_(.*?)$/, 1].downcase.to_sym
    foundElement.send_keys(key_convert)
  end
end

# method to select option from dropdwon list
def select_option_from_dropdown(element, by, option)
  dropdown = WAIT.until { find_object(element) }
  select_list = Selenium::WebDriver::Support::Select.new(dropdown)
  select_list.select_by(:"#{by}", "#{option}")
end

# method to select all option from dropdwon list
def select_all_option_from_multiselect_dropdown(element)
  dropdown = WAIT.until { find_object(element) }
  select_list = Selenium::WebDriver::Support::Select.new(dropdown)
  select_list.select_all
end

# method to unselect all option from dropdwon list
def unselect_all_option_from_multiselect_dropdown(element)
  dropdown = WAIT.until { find_object(element) }
  select_list = Selenium::WebDriver::Support::Select.new(dropdown)
  select_list.deselect_all
end

# method to check checkbox
def check(element)
  checkbox = find_object(element)
  checkbox.click unless checkbox.selected?
end

# method to uncheck checkbox
def uncheck(element)
  checkbox = find_object(element)
  checkbox.click if checkbox.selected?
end

# method to toggle checkbox checked
def toggle_checkbox(element)
  find_object(element).click
end

# method to select radio button
def select_radio_button(element)
  radio_button = WAIT.until { find_object(element) }
  radio_button.click unless radio_button.selected?
end

# method to select option from radio button group
def select_option_from_radio_button_group(element, by, option)
  radio_button_group = WAIT.until { find_object(element) }

  getter = ->(rb, by) { by == 'value' ? rb.attribute('value') : rb.text }
  ele = radio_button_group.find { |rb| getter.call(rb, by) == option }
  ele.click unless ele.selected?
end

def edit_content(element, text)
  value = check_is_using_variable(text)
  found_element = find_object(element)
  $driver.execute_script("if (arguments[0].contentEditable === 'true') {arguments[0].innerText = '#{value}'}", found_element)
end

def add_selection(element, label)
  value = check_is_using_variable(label)
  found_element = find_object(element)
  found_element.find_element(:xpath, "//option[. = '#{value}']").click
end

def remove_selection(element, label)
  value = check_is_using_variable(label)
  found_element = find_object(element)
  found_element.find_element(:xpath, "//option[. = '#{value}']").click
end

def select(element)
  found_element = find_object(element)
  found_element.find_element(:xpath, "//option[. = '#{label}']").click
end

def drag_and_drop_to_object(element, drop_element)
  dragged = find_object(element)
  object_value_regex = /^(class=|css=|id=|name=|xpath=|className=|link=|linkText=|partialLinkText=)/
  object_type_regex = /^(class|css|id|name|xpath|className|link|linkText|partialLinkText)/
  if object_value_regex.match?(drop_element)
    object_value = drop_element.gsub(object_value_regex, "")
    object_type = drop_element.split(object_type_regex)[1].underscore
    dropped = $driver.find_element(object_type.to_sym, object_value)
    $driver.action.drag_and_drop(dragged, dropped).perform
  else
    raise TestCaseFailed, "Option locator not supported"
  end
end

def select_window(element)
end

def select_frame(element)
end
