require 'selenium-webdriver'

# custome exception class
class TestCaseFailed < Exception
end

# WAIT instance for explicit wait
WAIT = Selenium::WebDriver::Wait.new(:timeout => 30)

# method to validate locator
def valid_locator_type? type
  %w(id class css name xpath class_name link link_text partial_link_text).include? type
end

def validate_locator type
  raise "Invalid locator type - #{type}" unless valid_locator_type? type
end

# method to validate dropdown selector
def valid_option_by? option_by
  %w(text value index).include? option_by
end

def validate_option_by option_by
  raise "Invalid option by - #{option_by}" unless valid_option_by? option_by
end

# Return android device name and android version using adb command
def get_device_info
  IO.popen('adb shell getprop ro.product.brand') { |f| $device = f.gets.chomp.upcase}
  $device += ' '
  IO.popen('adb shell getprop ro.product.model') { |f| $device += f.gets.chomp.upcase}
  IO.popen('adb shell getprop ro.build.version.release') { |f| $os_version = f.gets.chomp.upcase}
  return $device, $os_version
end

def find_object(string_object)
  $OBJECT = load_object_repository
  string_object = string_object.gsub(/"/, "'")
  hash_object = $OBJECT[string_object]
  if hash_object == nil
    raise "Object not found - Object: #{string_object} is not found in Object Repository."
  end
  locator_type = hash_object.keys.first.to_s
  locator_value = hash_object[locator_type]
  validate_locator(locator_type)
  $driver.find_element(:"#{locator_type}" => "#{locator_value}")
end

def load_object_repository
  begin
    loadYAMLfile("features/step_definitions/repositories/project_object.yml")
  rescue => exception
    print_error_object_repository
  end
end

def loadYAMLfile(filename)
  begin
    return YAML.load_file(filename)
  rescue Psych::SyntaxError => ex
    raise "Syntax error when reading file: #{ex}"
  end
end

def check_using_variable(text)
  regex = /\$\{(.*?)\}/
  text.scan(regex).each do |var|
    value = $vars[var[0].to_sym]
    if value
      replace = "${#{var[0]}}"
      text.gsub!(replace, "#{value}")
    end
  end
  text
end
