require_relative 'required_files'

# method to open link
def open(link)
  value = check_is_using_variable(link)
  $driver.get(value)
end

# method to navigate back & forward
def navigate(direction)
  if direction == 'back'
    $driver.navigate.back
  else
    $driver.navigate.forward
  end
end

def close
  $drive.close
end

# method to quite webdriver instance
def quit
  $driver.quit
end

# method to return key by os wise
def get_key
  os = Selenium::WebDriver::Platform.os
  if os.to_s == 'windows'
    return 'control'
  elsif os.to_s == 'macosx'
    return 'command'
  else
    raise 'Invalid OS'
  end
end

# Method to zoom in/out page
def zoom_in_out(in_out)
  $driver.action.key_down(:"#{get_key}").send_keys(:"#{in_out}").key_up(:"#{get_key}").perform
end

# Method to zoom in/out web page until web element dislays
def zoom_in_out_till_element_display(element, in_out)
  while true
    if find_object(element).displayed?
      break
    else
      $driver.action.key_down(:"#{get_key}").send_keys(:"#{in_out}").key_up(:"#{get_key}").perform
    end
  end
end

# Method to resize browser
def set_window_size(resolution)
  value = check_is_using_variable(resolution)
  raise TestCaseFailed, 'Wrong format of resolution. (e.g., 1280x800)' unless value.match(/^\d+x\d+$/)
  width, height = value.split("x")
  $driver.manage.resize_to(width, heigth)
end

# Method to maximize browser
def maximize_browser
  $driver.manage.window.maximize
end

# Method to refresh page
def refresh_page
  $driver.navigate.refresh
end

# Method to hover on element
def mouse_over(element)
  element = find_object(element)
  $driver.action.move_to_element(element).perform
end

# mouse_down
def mouse_down(element)
  element = find_object(element)
  $driver.action.move_to_element(element).click_and_hold.perform
end

# mouse_down_at
def mouse_down_at(element, location)
  element = find_object(element)
  $driver.action.move_to_element(element).click_and_hold.perform
end

# mouse_out
def mouse_out
  element = $driver.find_element(By.CSS_SELECTOR, 'body')
  $driver.action.move_to_element(element, 0, 0).perform
end

#mouse_up
def mouse_up(element)
  element = find_object(element)
  $driver.action.move_to_element(element).release.perform
end

#mouse_up_at
def mouse_up_at
  element = find_object(element)
  $driver.action.move_to_element(element).release.perform
end

#mouse_move_at
def mouse_move_at
  element = find_object(element)
  $driver.action.move_to_element(element).perform
end

# Method to scroll page to perticular element
def scroll_to_element(element)
  ele_scroll = find_object(element)
  ele_scroll.location_once_scrolled_into_view
end

# method to scroll page to top or end
def scroll_page(to)
  if to == 'end'
    $driver.execute_script('window.scrollTo(0,Math.max(document.documentElement.scrollHeight,document.body.scrollHeight,document.documentElement.clientHeight));')
  elsif to == 'top'
    $driver.execute_script('window.scrollTo(Math.max(document.documentElement.scrollHeight,document.body.scrollHeight,document.documentElement.clientHeight),0);')
  else
    raise "Exception : Invalid Direction (only scroll \"top\" or \"end\")"
  end
end
